#import <UIKit/UIKit.h>

#define NCELLS 4

@interface HelloController : UITableViewController 
{
	NSMutableArray *tableTitles;
	int ithTitle;
}
@end


@implementation HelloController

- (HelloController *) init
{
	if (!(self = [super init])) return self;
	
	self.title = @"Table Edits";
	tableTitles = [[NSMutableArray alloc] init];
	ithTitle = NCELLS;
	for (int i = 1; i <= NCELLS; i++) [tableTitles addObject:[NSString stringWithFormat:@"Table Cell #%d", i]];
	
	return self;
}

#pragma mark UITableViewDataSource Methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView 
{
	return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section 
{
	return [tableTitles count];
}

// Return a cell for the ith row
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"any-cell"];
	if (!cell) cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:@"any-cell"] autorelease];

	cell.text = [tableTitles objectAtIndex:[indexPath row]];
	cell.showsReorderControl = YES; 
	return cell;
}

- (void) add
{
	[tableTitles addObject:[NSString stringWithFormat:@"Table Cell #%d", ++ithTitle]];	
	[self.tableView reloadData];
}

#pragma mark UITableViewDelegateMethods
- (void) deselect
{	
	[self.tableView deselectRowAtIndexPath:[self.tableView indexPathForSelectedRow] animated:YES];
}

// Perform the re-order
-(void) tableView: (UITableView *) tableView moveRowAtIndexPath: (NSIndexPath *) oldPath toIndexPath:(NSIndexPath *) newPath
{
	NSString *title = [tableTitles objectAtIndex:[oldPath row]];
	[tableTitles removeObjectAtIndex:[oldPath row]];
	[tableTitles insertObject:title atIndex:[newPath row]];
}

// Respond to user selection
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)newIndexPath
{
	printf("User selected row %d\n", [newIndexPath row] + 1);
	[self performSelector:@selector(deselect) withObject:nil afterDelay:0.5f];
}

// Handle deletion
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle 
forRowAtIndexPath:(NSIndexPath *)indexPath 
{
	printf("About to delete item %d\n", [indexPath row]);
	[tableTitles removeObjectAtIndex:[indexPath row]];
	[self.tableView reloadData];
}	 

- (void)loadView
{
	[super loadView];

	// Add an add button
	self.navigationItem.leftBarButtonItem = [[[UIBarButtonItem alloc]
											  initWithTitle:@"Add" 
											  style:UIBarButtonItemStylePlain 
											  target:self 
											  action:@selector(add)] autorelease];
	
	// Add the edit button
	self.navigationItem.rightBarButtonItem = self.editButtonItem;
}

-(void) dealloc
{
	[tableTitles release];
	[super dealloc];
}
@end

@interface SampleAppDelegate : NSObject <UIApplicationDelegate> 
{
	UINavigationController *nav;
}
@property (nonatomic, retain)		UINavigationController *nav;
@end

@implementation SampleAppDelegate
@synthesize nav;
- (void)applicationDidFinishLaunching:(UIApplication *)application {	
	UIWindow *window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
	self.nav = [[UINavigationController alloc] initWithRootViewController:[[HelloController alloc] init]];
	[window addSubview:self.nav.view];
	[window makeKeyAndVisible];
}

- (void) dealloc
{
	[self.nav release];
	[super dealloc];
}
@end

int main(int argc, char *argv[])
{
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
	int retVal = UIApplicationMain(argc, argv, nil, @"SampleAppDelegate");
	[pool release];
	return retVal;
}
